@@ -140,30 +140,31 @@ _arg_cache_new_for_interface (GIBaseInfo *iface_info, GITypeInfo *type_info,
                               PyGIDirection direction,
                               PyGICallableCache *callable_cache)
 {
-    if (GI_IS_CALLBACK_INFO (iface_info)) {
+    switch (pygi_interface_type_tag (iface_info)) {
+    case PYGI_INTERFACE_TYPE_TAG_CALLBACK:
         return pygi_arg_callback_new_from_info (
             type_info, arg_info, transfer, direction,
             GI_CALLBACK_INFO (iface_info), callable_cache);
-    } else if (GI_IS_OBJECT_INFO (iface_info)
-               || GI_IS_INTERFACE_INFO (iface_info)) {
+    case PYGI_INTERFACE_TYPE_TAG_OBJECT:
+    case PYGI_INTERFACE_TYPE_TAG_INTERFACE:
         return pygi_arg_gobject_new_from_info (
             type_info, arg_info, transfer, direction,
             GI_REGISTERED_TYPE_INFO (iface_info), callable_cache);
-    } else if (GI_IS_STRUCT_INFO (iface_info)
-               || GI_IS_UNION_INFO (iface_info)) {
+    case PYGI_INTERFACE_TYPE_TAG_STRUCT:
+    case PYGI_INTERFACE_TYPE_TAG_UNION:
         return pygi_arg_struct_new_from_info (
             type_info, arg_info, transfer, direction,
             GI_REGISTERED_TYPE_INFO (iface_info));
-    } else if (GI_IS_FLAGS_INFO (iface_info)) {
+    case PYGI_INTERFACE_TYPE_TAG_FLAGS:
         /* Check flags before enums: flags are a subtype of enum. */
         return pygi_arg_flags_new_from_info (type_info, arg_info, transfer,
                                              direction,
                                              GI_FLAGS_INFO (iface_info));
-    } else if (GI_IS_ENUM_INFO (iface_info)) {
+    case PYGI_INTERFACE_TYPE_TAG_ENUM:
         return pygi_arg_enum_new_from_info (type_info, arg_info, transfer,
                                             direction,
                                             GI_ENUM_INFO (iface_info));
-    } else {
+    default:
         g_assert_not_reached ();
     }
 
@@ -184,6 +185,9 @@ pygi_arg_cache_new (GITypeInfo *type_info,
 
     switch (type_tag) {
     case GI_TYPE_TAG_VOID:
+        arg_cache = pygi_arg_void_type_new_from_info (type_info, arg_info,
+                                                      transfer, direction);
+        break;
     case GI_TYPE_TAG_BOOLEAN:
     case GI_TYPE_TAG_INT8:
     case GI_TYPE_TAG_UINT8:
@@ -197,12 +201,14 @@ pygi_arg_cache_new (GITypeInfo *type_info,
     case GI_TYPE_TAG_DOUBLE:
     case GI_TYPE_TAG_UNICHAR:
     case GI_TYPE_TAG_GTYPE:
+        arg_cache = pygi_arg_numeric_type_new_from_info (type_info, arg_info,
+                                                         transfer, direction);
+        break;
     case GI_TYPE_TAG_UTF8:
     case GI_TYPE_TAG_FILENAME:
-        arg_cache = pygi_arg_basic_type_new_from_info (type_info, arg_info,
-                                                       transfer, direction);
+        arg_cache = pygi_arg_string_type_new_from_info (type_info, arg_info,
+                                                        transfer, direction);
         break;
-
     case GI_TYPE_TAG_ARRAY:
         arg_cache = pygi_arg_garray_new_from_info (
             type_info, arg_info, transfer, direction, callable_cache,
@@ -238,7 +244,7 @@ pygi_arg_cache_new (GITypeInfo *type_info,
                                                    transfer, direction);
         break;
     default:
-        break;
+        g_assert_not_reached ();
     }
 
     if (arg_cache != NULL) {
@@ -742,7 +748,6 @@ pygi_function_cache_invoke (PyGIFunctionCache *function_cache,
     PyGIInvokeState state = {
         0,
     };
-
     return function_cache->invoke (function_cache, &state, py_args, py_nargsf,
                                    py_kwnames);
 }
