@@ -1117,7 +1117,7 @@ _pygi_g_type_tag_size (GITypeTag type_tag)
                       gi_type_tag_to_string (type_tag));
         break;
     default:
-        break;
+        g_assert_not_reached ();
     }
 
     return size;
@@ -1153,19 +1153,23 @@ _pygi_gi_type_info_size (GITypeInfo *type_info)
 
         info = gi_type_info_get_interface (type_info);
 
-        if (GI_IS_STRUCT_INFO (info)) {
+        switch (pygi_interface_type_tag (info)) {
+        case PYGI_INTERFACE_TYPE_TAG_STRUCT:
             if (gi_type_info_is_pointer (type_info)) {
                 size = sizeof (gpointer);
             } else {
                 size = gi_struct_info_get_size ((GIStructInfo *)info);
             }
-        } else if (GI_IS_UNION_INFO (info)) {
+            break;
+        case PYGI_INTERFACE_TYPE_TAG_UNION:
             if (gi_type_info_is_pointer (type_info)) {
                 size = sizeof (gpointer);
             } else {
                 size = gi_union_info_get_size ((GIUnionInfo *)info);
             }
-        } else if (GI_IS_ENUM_INFO (info)) {
+            break;
+        case PYGI_INTERFACE_TYPE_TAG_ENUM:
+        case PYGI_INTERFACE_TYPE_TAG_FLAGS:
             if (gi_type_info_is_pointer (type_info)) {
                 size = sizeof (gpointer);
             } else {
@@ -1175,10 +1179,13 @@ _pygi_gi_type_info_size (GITypeInfo *type_info)
                     gi_enum_info_get_storage_type ((GIEnumInfo *)info);
                 size = _pygi_g_type_tag_size (enum_type_tag);
             }
-        } else if (GI_IS_OBJECT_INFO (info) || GI_IS_INTERFACE_INFO (info)
-                   || GI_IS_CALLBACK_INFO (info)) {
+            break;
+        case PYGI_INTERFACE_TYPE_TAG_OBJECT:
+        case PYGI_INTERFACE_TYPE_TAG_INTERFACE:
+        case PYGI_INTERFACE_TYPE_TAG_CALLBACK:
             size = sizeof (gpointer);
-        } else {
+            break;
+        default:
             g_assert_not_reached ();
         }
 
@@ -1196,7 +1203,7 @@ _pygi_gi_type_info_size (GITypeInfo *type_info)
         size = sizeof (gpointer);
         break;
     default:
-        break;
+        g_assert_not_reached ();
     }
 
     return size;
@@ -1439,24 +1446,31 @@ pygi_gi_struct_info_is_simple (GIStructInfo *struct_info)
 
             info = gi_type_info_get_interface (field_type_info);
 
-            if (GI_IS_STRUCT_INFO (info)) {
+            switch (pygi_interface_type_tag (info)) {
+            case PYGI_INTERFACE_TYPE_TAG_STRUCT:
                 if (gi_type_info_is_pointer (field_type_info)) {
                     is_simple = FALSE;
                 } else {
                     is_simple =
                         pygi_gi_struct_info_is_simple ((GIStructInfo *)info);
                 }
-            } else if (GI_IS_UNION_INFO (info)) {
+                break;
+            case PYGI_INTERFACE_TYPE_TAG_UNION:
                 /* TODO */
                 is_simple = FALSE;
-            } else if (GI_IS_ENUM_INFO (info)) {
+                break;
+            case PYGI_INTERFACE_TYPE_TAG_ENUM:
+            case PYGI_INTERFACE_TYPE_TAG_FLAGS:
                 if (gi_type_info_is_pointer (field_type_info)) {
                     is_simple = FALSE;
                 }
-            } else if (GI_IS_OBJECT_INFO (info) || GI_IS_CALLBACK_INFO (info)
-                       || GI_IS_INTERFACE_INFO (info)) {
+                break;
+            case PYGI_INTERFACE_TYPE_TAG_OBJECT:
+            case PYGI_INTERFACE_TYPE_TAG_INTERFACE:
+            case PYGI_INTERFACE_TYPE_TAG_CALLBACK:
                 is_simple = FALSE;
-            } else {
+                break;
+            default:
                 g_assert_not_reached ();
             }
 
@@ -1465,7 +1479,6 @@ pygi_gi_struct_info_is_simple (GIStructInfo *struct_info)
         }
         default:
             g_assert_not_reached ();
-            break;
         }
 
         gi_base_info_unref ((GIBaseInfo *)field_type_info);
@@ -1823,7 +1836,7 @@ static PyObject *
 _wrap_gi_constant_info_get_value (PyGIBaseInfo *self)
 {
     GITypeInfo *type_info;
-    GIArgument value = { 0 };
+    GIArgument value = PYGI_ARG_INIT;
     PyObject *py_value;
     gboolean free_array = FALSE;
 
@@ -1832,12 +1845,12 @@ _wrap_gi_constant_info_get_value (PyGIBaseInfo *self)
     type_info = gi_constant_info_get_type_info ((GIConstantInfo *)self->info);
 
     if (gi_type_info_get_tag (type_info) == GI_TYPE_TAG_ARRAY) {
-        value.v_pointer = _pygi_argument_to_array (&value, NULL, NULL, NULL,
+        value.v_pointer = _pygi_argument_to_array (value, NULL, NULL, NULL,
                                                    type_info, &free_array);
     }
 
     py_value =
-        _pygi_argument_to_object (&value, type_info, GI_TRANSFER_NOTHING);
+        _pygi_argument_to_object (value, type_info, GI_TRANSFER_NOTHING);
 
     if (free_array) {
         g_array_free (value.v_pointer, FALSE);
@@ -1878,14 +1891,14 @@ static PyMethodDef _PyGIValueInfo_methods[] = {
 /* GIFieldInfo */
 PYGI_DEFINE_TYPE ("gi.FieldInfo", PyGIFieldInfo_Type, PyGIBaseInfo);
 
-static gssize
+static gboolean
 _struct_field_array_length_marshal (gsize length_index, void *container_ptr,
-                                    void *struct_data_ptr)
+                                    void *struct_data_ptr, gsize *array_len)
 {
-    gssize array_len = -1;
     GIFieldInfo *array_len_field = NULL;
-    GIArgument arg = { 0 };
+    GIArgument arg = PYGI_ARG_INIT;
     GIBaseInfo *container_info = (GIBaseInfo *)container_ptr;
+    gboolean result = TRUE;
 
     if (GI_IS_UNION_INFO (container_info)) {
         array_len_field = gi_union_info_get_field (
@@ -1902,7 +1915,7 @@ _struct_field_array_length_marshal (gsize length_index, void *container_ptr,
     }
 
     if (array_len_field == NULL) {
-        return -1;
+        return FALSE;
     }
 
     if (gi_field_info_get_field (array_len_field, struct_data_ptr, &arg)) {
@@ -1913,10 +1926,9 @@ _struct_field_array_length_marshal (gsize length_index, void *container_ptr,
             goto out;
         }
 
-        if (!pygi_argument_to_gssize (
-                &arg, gi_type_info_get_tag (array_len_type_info),
-                &array_len)) {
-            array_len = -1;
+        if (!pygi_argument_to_gsize (
+                arg, gi_type_info_get_tag (array_len_type_info), array_len)) {
+            result = FALSE;
         }
 
         gi_base_info_unref (array_len_type_info);
@@ -1924,7 +1936,7 @@ _struct_field_array_length_marshal (gsize length_index, void *container_ptr,
 
 out:
     gi_base_info_unref (array_len_field);
-    return array_len;
+    return result;
 }
 
 static gint
@@ -2050,13 +2062,14 @@ _wrap_gi_field_info_get_value (PyGIBaseInfo *self, PyObject *args)
 
         info = gi_type_info_get_interface (field_type_info);
 
-        if (GI_IS_UNION_INFO (info)) {
+        switch (pygi_interface_type_tag (info)) {
+        case PYGI_INTERFACE_TYPE_TAG_UNION:
             PyErr_SetString (PyExc_NotImplementedError,
                              "getting an union is not supported yet");
             gi_base_info_unref (info);
 
             goto out;
-        } else if (GI_IS_STRUCT_INFO (info)) {
+        case PYGI_INTERFACE_TYPE_TAG_STRUCT: {
             gsize offset;
 
             offset = gi_field_info_get_offset ((GIFieldInfo *)self->info);
@@ -2066,10 +2079,16 @@ _wrap_gi_field_info_get_value (PyGIBaseInfo *self, PyObject *args)
             gi_base_info_unref (info);
 
             goto argument_to_object;
-        } else {
+        }
+        case PYGI_INTERFACE_TYPE_TAG_ENUM:
+        case PYGI_INTERFACE_TYPE_TAG_FLAGS:
+        case PYGI_INTERFACE_TYPE_TAG_OBJECT:
+        case PYGI_INTERFACE_TYPE_TAG_INTERFACE:
+        case PYGI_INTERFACE_TYPE_TAG_CALLBACK:
             gi_base_info_unref (info);
-
-            /* Fallback. */
+            break;
+        default:
+            g_assert_not_reached ();
         }
     }
 
@@ -2081,13 +2100,13 @@ _wrap_gi_field_info_get_value (PyGIBaseInfo *self, PyObject *args)
 
     if (gi_type_info_get_tag (field_type_info) == GI_TYPE_TAG_ARRAY) {
         value.v_pointer = _pygi_argument_to_array (
-            &value, _struct_field_array_length_marshal, container_info,
-            pointer, field_type_info, &free_array);
+            value, _struct_field_array_length_marshal, container_info, pointer,
+            field_type_info, &free_array);
     }
 
 argument_to_object:
-    py_value = _pygi_argument_to_object (&value, field_type_info,
-                                         GI_TRANSFER_NOTHING);
+    py_value =
+        _pygi_argument_to_object (value, field_type_info, GI_TRANSFER_NOTHING);
 
     if (free_array) {
         g_array_free (value.v_pointer, FALSE);
