@@ -20,6 +20,7 @@
  * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "pygi-argument.h"
 #include "pygi-async.h"
 #include "pygi-boxed.h"
 #include "pygi-error.h"
@@ -360,7 +361,7 @@ _caller_alloc (PyGIArgCache *arg_cache, GIArgument *arg)
             arg->v_pointer = g_byte_array_new ();
             break;
         default:
-            return FALSE;
+            g_assert_not_reached ();
         }
     } else {
         return FALSE;
@@ -534,7 +535,6 @@ _invoke_marshal_in_args (PyGIInvokeState *state,
             break;
         default:
             g_assert_not_reached ();
-            break;
         }
 
         if (py_arg == _PyGIDefaultArgPlaceholder) {
@@ -706,7 +706,7 @@ pygi_invoke_c_callable (PyGIFunctionCache *function_cache,
                         size_t py_nargsf, PyObject *py_kwnames)
 {
     PyGICallableCache *cache = (PyGICallableCache *)function_cache;
-    GIFFIReturnValue ffi_return_value = { 0 };
+    GIFFIReturnValue ffi_return_value = PYGI_ARG_INIT;
     PyObject *ret = NULL;
 
     if (Py_EnterRecursiveCall (" while calling a GICallable")) return NULL;
