@@ -22,6 +22,7 @@
  */
 
 #include "pygi-argument.h"
+#include "pygi-basictype.h"
 #include "pygi-fundamental.h"
 #include "pygi-property.h"
 #include "pygi-repository.h"
@@ -123,7 +124,6 @@ pygi_get_property_value (PyGObject *instance, GParamSpec *pspec)
     };
     PyObject *py_value = NULL;
     GType fundamental;
-    gboolean handled;
 
     if (!(pspec->flags & G_PARAM_READABLE)) {
         PyErr_Format (PyExc_TypeError, "property %s is not readable",
@@ -140,13 +140,18 @@ pygi_get_property_value (PyGObject *instance, GParamSpec *pspec)
     Py_BEGIN_ALLOW_THREADS;
     g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
     g_object_get_property (instance->obj, pspec->name, &value);
-    fundamental = G_TYPE_FUNDAMENTAL (G_VALUE_TYPE (&value));
     Py_END_ALLOW_THREADS;
 
+    // special case: unichar, which has a uint type
+    if (G_IS_PARAM_SPEC_UNICHAR (pspec)) {
+        py_value = pygi_gunichar_to_py (g_value_get_uint (&value));
+        goto out;
+    }
 
-    /* Fast path basic types which don't need GI type info. */
-    py_value = pygi_value_to_py_basic_type (&value, fundamental, &handled);
-    if (handled) {
+    /* Fast path: basic types which don't need GI type info. */
+    fundamental = G_TYPE_FUNDAMENTAL (G_VALUE_TYPE (&value));
+    py_value = pygi_value_to_py_basic_type (&value, fundamental);
+    if (py_value != NULL || PyErr_Occurred ()) {
         goto out;
     }
 
@@ -158,9 +163,7 @@ pygi_get_property_value (PyGObject *instance, GParamSpec *pspec)
     if (property_info) {
         GITypeInfo *type_info = NULL;
         gboolean free_array = FALSE;
-        GIArgument arg = {
-            0,
-        };
+        GIArgument arg;
         GITransfer transfer = GI_TRANSFER_NOTHING;
 
         type_info = gi_property_info_get_type_info (property_info);
@@ -168,14 +171,14 @@ pygi_get_property_value (PyGObject *instance, GParamSpec *pspec)
 
         /* Arrays are special cased, see note in _pygi_argument_to_array. */
         if (gi_type_info_get_tag (type_info) == GI_TYPE_TAG_ARRAY) {
-            arg.v_pointer = _pygi_argument_to_array (&arg, NULL, NULL, NULL,
+            arg.v_pointer = _pygi_argument_to_array (arg, NULL, NULL, NULL,
                                                      type_info, &free_array);
         } else if (g_type_is_a (pspec->value_type, G_TYPE_BOXED)) {
             arg.v_pointer = g_value_dup_boxed (&value);
             transfer = GI_TRANSFER_EVERYTHING;
         }
 
-        py_value = _pygi_argument_to_object (&arg, type_info, transfer);
+        py_value = _pygi_argument_to_object (arg, type_info, transfer);
 
         if (free_array) {
             g_array_free (arg.v_pointer, FALSE);
@@ -191,7 +194,7 @@ pygi_get_property_value (PyGObject *instance, GParamSpec *pspec)
 
     /* Fallback to GValue marshalling. */
     if (py_value == NULL) {
-        py_value = pyg_param_gvalue_as_pyobject (&value, TRUE, pspec);
+        py_value = pyg_value_to_pyobject (&value, TRUE);
     }
 
 out:
@@ -221,12 +224,9 @@ pygi_set_property_gvalue_from_property_info (GIPropertyInfo *property_info,
                                              GValue *value, PyObject *py_value)
 {
     GITypeInfo *type_info = NULL;
-    GIBaseInfo *info = NULL;
     GITypeTag type_tag;
     GITransfer transfer;
-    GIArgument arg = {
-        0,
-    };
+    GIArgument arg;
     gint ret_value = -1;
 
     type_info = gi_property_info_get_type_info (property_info);
@@ -239,18 +239,19 @@ pygi_set_property_gvalue_from_property_info (GIPropertyInfo *property_info,
     type_tag = gi_type_info_get_tag (type_info);
     switch (type_tag) {
     case GI_TYPE_TAG_INTERFACE: {
-        GType type;
+        GIBaseInfo *interface = gi_type_info_get_interface (type_info);
+        GType type = gi_registered_type_info_get_g_type (
+            GI_REGISTERED_TYPE_INFO (interface));
 
-        info = gi_type_info_get_interface (type_info);
-        type = gi_registered_type_info_get_g_type (
-            GI_REGISTERED_TYPE_INFO (info));
-
-        if (GI_IS_FLAGS_INFO (info)) {
-            /* Check flags before enums: flags are a subtype of enum. */
+        switch (pygi_interface_type_tag (interface)) {
+        case PYGI_INTERFACE_TYPE_TAG_FLAGS:
             g_value_set_flags (value, arg.v_uint);
-        } else if (GI_IS_ENUM_INFO (info)) {
+            break;
+        case PYGI_INTERFACE_TYPE_TAG_ENUM:
             g_value_set_enum (value, arg.v_int);
-        } else if (GI_IS_INTERFACE_INFO (info) || GI_IS_OBJECT_INFO (info)) {
+            break;
+        case PYGI_INTERFACE_TYPE_TAG_INTERFACE:
+        case PYGI_INTERFACE_TYPE_TAG_OBJECT:
             if (arg.v_pointer == NULL || G_IS_OBJECT (arg.v_pointer)) {
                 g_value_set_object (value, arg.v_pointer);
             } else if (!pygi_fundamental_set_value (value, arg.v_pointer)) {
@@ -260,7 +261,9 @@ pygi_set_property_gvalue_from_property_info (GIPropertyInfo *property_info,
                     g_type_name (type));
                 goto out;
             }
-        } else if (GI_IS_STRUCT_INFO (info) || GI_IS_UNION_INFO (info)) {
+            break;
+        case PYGI_INTERFACE_TYPE_TAG_STRUCT:
+        case PYGI_INTERFACE_TYPE_TAG_UNION:
             if (g_type_is_a (type, G_TYPE_BOXED)) {
                 g_value_set_boxed (value, arg.v_pointer);
             } else if (g_type_is_a (type, G_TYPE_VARIANT)) {
@@ -272,12 +275,17 @@ pygi_set_property_gvalue_from_property_info (GIPropertyInfo *property_info,
                     g_type_name (type));
                 goto out;
             }
-        } else {
+            break;
+        case PYGI_INTERFACE_TYPE_TAG_CALLBACK:
             PyErr_Format (PyExc_NotImplementedError,
                           "Setting properties of type '%s' is not implemented",
                           g_type_name (type));
             goto out;
+        default:
+            g_assert_not_reached ();
         }
+        gi_base_info_unref (interface);
+
         break;
     }
     case GI_TYPE_TAG_BOOLEAN:
@@ -304,6 +312,7 @@ pygi_set_property_gvalue_from_property_info (GIPropertyInfo *property_info,
         break;
     case GI_TYPE_TAG_UINT16:
     case GI_TYPE_TAG_UINT32:
+    case GI_TYPE_TAG_UNICHAR:
         if (G_VALUE_HOLDS_ULONG (value))
             g_value_set_ulong (value, arg.v_ulong);
         else
@@ -356,19 +365,22 @@ pygi_set_property_gvalue_from_property_info (GIPropertyInfo *property_info,
         g_array_free (arg_items, TRUE);
         break;
     }
-    default:
+    case GI_TYPE_TAG_VOID:
+    case GI_TYPE_TAG_GSLIST:
+    case GI_TYPE_TAG_ERROR:
         PyErr_Format (
             PyExc_NotImplementedError,
             "Setting properties of type %s is not implemented",
             gi_type_tag_to_string (gi_type_info_get_tag (type_info)));
         goto out;
+    default:
+        g_assert_not_reached ();
     }
 
     ret_value = 0;
 
 out:
     if (type_info != NULL) gi_base_info_unref (type_info);
-    if (info != NULL) gi_base_info_unref (info);
 
     return ret_value;
 }
@@ -428,7 +440,15 @@ pygi_set_property_value (PyGObject *instance, GParamSpec *pspec,
         goto out;
     }
 
-    if (pygi_set_gvalue_for_pspec (pspec, &value, py_value) < 0) {
+    // special case: unichar has an internal type uint
+    if (G_IS_PARAM_SPEC_UNICHAR (pspec)) {
+        gunichar u;
+
+        if (!pygi_gunichar_from_py (py_value, &u)) {
+            goto out;
+        }
+        g_value_set_uint (&value, u);
+    } else if (pygi_set_gvalue_for_pspec (pspec, &value, py_value) < 0) {
         /* If we already have an error set, don't override it,
          * otherwise raise a TypError indcating that we couldn't
          * set the property */
