@@ -126,7 +126,6 @@ generic_gsize_richcompare (gsize a, gsize b, int op)
         Py_INCREF (res);
         break;
 
-
     case Py_LT:
         res = (a < b) ? Py_True : Py_False;
         Py_INCREF (res);
@@ -148,8 +147,7 @@ generic_gsize_richcompare (gsize a, gsize b, int op)
         break;
 
     default:
-        res = Py_NewRef (Py_NotImplemented);
-        break;
+        g_assert_not_reached ();
     }
 
     return res;
@@ -830,7 +828,7 @@ pyg_closure_marshal (GClosure *closure, GValue *return_value,
             Py_INCREF (pc->swap_data);
             PyTuple_SetItem (params, 0, pc->swap_data);
         } else {
-            PyObject *item = pyg_value_as_pyobject (&param_values[i], FALSE);
+            PyObject *item = pyg_value_to_pyobject (&param_values[i], FALSE);
 
             /* error condition */
             if (!item) {
@@ -988,7 +986,7 @@ pyg_signal_class_closure_marshal (GClosure *closure, GValue *return_value,
        initially because we'll check after the call to see if a copy is needed. */
     params = PyTuple_New (n_param_values - 1);
     for (i = 1; i < n_param_values; i++) {
-        PyObject *item = pyg_value_as_pyobject (&param_values[i], FALSE);
+        PyObject *item = pyg_value_to_pyobject (&param_values[i], FALSE);
 
         /* error condition */
         if (!item) {
@@ -1321,3 +1319,39 @@ pygi_type_register_types (PyObject *d)
 
     return 0;
 }
+
+/**
+ * pygi_interface_type_tag:
+ *
+ * A simple way to map types often obtained by calling
+ * `gi_type_info_get_interface` to an enum.
+ * This makes for easier to read code.
+ */
+PyGIInterfaceTypeTag
+pygi_interface_type_tag (GIBaseInfo *info)
+{
+    PyGIInterfaceTypeTag iface_type_tag;
+
+    if (GI_IS_FLAGS_INFO (info)) {
+        /* Check flags before enums: flags are a subtype of enum. */
+        iface_type_tag = PYGI_INTERFACE_TYPE_TAG_FLAGS;
+    } else if (GI_IS_ENUM_INFO (info)) {
+        iface_type_tag = PYGI_INTERFACE_TYPE_TAG_ENUM;
+    } else if (GI_IS_INTERFACE_INFO (info)) {
+        iface_type_tag = PYGI_INTERFACE_TYPE_TAG_INTERFACE;
+    } else if (GI_IS_OBJECT_INFO (info)) {
+        iface_type_tag = PYGI_INTERFACE_TYPE_TAG_OBJECT;
+    } else if (GI_IS_STRUCT_INFO (info)) {
+        iface_type_tag = PYGI_INTERFACE_TYPE_TAG_STRUCT;
+    } else if (GI_IS_UNION_INFO (info)) {
+        iface_type_tag = PYGI_INTERFACE_TYPE_TAG_UNION;
+    } else if (GI_IS_CALLBACK_INFO (info)) {
+        iface_type_tag = PYGI_INTERFACE_TYPE_TAG_CALLBACK;
+    } else {
+        g_critical ("Type %s is not handled",
+                    g_type_name_from_instance ((GTypeInstance *)info));
+        g_assert_not_reached ();
+    }
+
+    return iface_type_tag;
+}
