@@ -18,13 +18,6 @@ from gi.repository import GObject
 from gi.repository import GLib
 from gi.repository import Gio
 
-try:
-    from gi.repository import Gtk
-
-    Gtk
-except ImportError:
-    Gtk = None
-
 from .helper import capture_exceptions
 
 
@@ -1434,7 +1427,6 @@ class TestTortureProfile(unittest.TestCase):
     def test_torture_profile(self):
         total_time = 0
         object_ = Everything.TestObj()
-        sys.stdout.write("\ttorture test 1 (10000 iterations): ")
 
         start_time = timeit.default_timer()
         for i in range(10000):
@@ -1444,8 +1436,6 @@ class TestTortureProfile(unittest.TestCase):
         delta_time = end_time - start_time
         total_time += delta_time
 
-        sys.stdout.write("\ttorture test 2 (10000 iterations): ")
-
         start_time = timeit.default_timer()
         for i in range(10000):
             (_y, _z, _q) = Everything.TestObj().torture_signature_0(
@@ -1456,7 +1446,6 @@ class TestTortureProfile(unittest.TestCase):
         delta_time = end_time - start_time
         total_time += delta_time
 
-        sys.stdout.write("\ttorture test 3 (10000 iterations): ")
         start_time = timeit.default_timer()
         for i in range(10000):
             with contextlib.suppress(Exception):
@@ -1467,8 +1456,6 @@ class TestTortureProfile(unittest.TestCase):
         delta_time = end_time - start_time
         total_time += delta_time
 
-        sys.stdout.write("\ttorture test 4 (10000 iterations): ")
-
         def callback(userdata):
             return 0
 
