@@ -1399,6 +1399,16 @@ class CPropertiesTestBase:
         self.set_prop(obj, "gtype", str)
         self.assertEqual(self.get_prop(obj, "gtype"), GObject.TYPE_STRING)
 
+    def test_unichar(self):
+        obj = Regress.TestObj()
+        self.assertEqual(self.get_prop(obj, "unichar"), "")
+
+        self.set_prop(obj, "unichar", "🙃")
+        self.assertEqual(self.get_prop(obj, "unichar"), "🙃")
+
+        with pytest.raises(TypeError):
+            self.set_prop(obj, "unichar", "🙃🙃🙃")
+
     def test_hash_table(self):
         obj = Regress.TestObj()
         self.assertEqual(self.get_prop(obj, "hash-table"), None)
